#-----------------------------------------------------------------------------
#
#   Loon - Unit missions
#
#-----------------------------------------------------------------------------

from unit import Unit
from unit_deposit import ResourceDeposit
import game
import resources

class Mission(object):
	#  target         unit or vector
	#  next_mission   Mission
	
	target = None
	next_mission = None
	description = "Untitled Mission"
	
	def get_goal_description(self):
		if self.next_mission:
			return self.next_mission.get_goal_description()
		else:
			return self.get_description()
	
	def get_description(self):
		return self.description
	
	def get_target_position(self):
		target = self.target
		if target is not None:
			if isinstance(target, Unit):
				return target.position
			else:
				return target
	
	def get_target_unit(self):
		target = self.target
		if isinstance(target, Unit):
			return target
	
	def get_target_resources(self):
		target = self.get_target_unit()
		if target:
			return target.resources
	
	def units_removed(self, unit, units):
		if self.target in units:
			self.target = None
			if self.next_mission:
				self.next_mission.units_removed(unit, units)

	def begin(self, unit):
		unit.mission = self

	def mission_accomplished(self, unit):
		unit.mission = self.next_mission
	
#-----------------------------------------------------------------------------

class TargetedMission(Mission):
	#  Automatically performs a MoveMission to the target first

	def __init__(self, target):
		self.target = target
	
	def get_description(self):
		target = self.target
		td = ""
		if isinstance(target, Unit):
			td = target.title + " at "
			tp = target.position
		else:
			tp = target
		return "%s %s(%.1f, %.1f)" % (self.description, td, tp[0], tp[1])

	def begin(self, unit):
		Mission.begin(self, unit)
		unit.push_mission(MoveMission(self.target))
	
	def carry_out(self, unit, dt):
		if self.target is None:
			self.mission_accomplished(unit)
		else:
			self.carry_out_on_target(unit, dt)

#-----------------------------------------------------------------------------

class MoveMission(TargetedMission):

	#description = "Move to"
	description = "Moving to"

	def carry_out_on_target(self, unit, dt):
		target = self.target
		if unit.move_towards(target, dt):
			self.mission_accomplished(unit)

#-----------------------------------------------------------------------------

class MineMission(TargetedMission):

	def get_description(self):
		target = self.target
		if isinstance(target, ResourceDeposit):
			#return "Mine %s at" % self.target.resource_type
			return "Mining %s at" % self.target.resource_type

	def carry_out_on_target(self, unit, dt):
		deposit = self.target
		if isinstance(deposit, ResourceDeposit):
			amount = min(dt * unit.mining_rate, deposit.health)
			deposit.health -= unit.resources.add(deposit.resource_type, amount)
			if deposit.health <= 0:
				game.game.world.remove_unit(deposit)
				self.mission_accomplished(unit)
			elif unit.resources.full_of(deposit.resource_type):
				self.mission_accomplished(unit)
		else:
			self.mission_accomplished(unit)

#-----------------------------------------------------------------------------

class GetMission(TargetedMission):

	#description = "Get resources from"
	description = "Getting resources from"
	
	def carry_out_on_target(self, unit, dt):
		res = self.get_target_resources()
		if res:
			res.transfer_all_to(unit.resources)
		self.mission_accomplished(unit)

#-----------------------------------------------------------------------------

class PutMission(TargetedMission):

	#description = "Put resources at"
	description = "Putting resources in"

	def carry_out_on_target(self, unit, dt):
		res = self.get_target_resources()
		if res:
			unit.resources.transfer_all_to(res)
		self.mission_accomplished(unit)

#-----------------------------------------------------------------------------

class BuildMission(TargetedMission):
	#  unit_class       class
	#  build_position   vector

	description = "Building"
	
	def __init__(self, target, unit, unit_class):
		self.build_position = target
		TargetedMission.__init__(self, target + (0, unit_class.radius + unit.radius))
		self.unit_class = unit_class
	
	def get_description(self):
		p = self.build_position
		return "%s %s at (%.1f, %.1f)" % (self.description, self.unit_class.title, p[0], p[1])
	
	def carry_out_on_target(self, unit, dt):
		pos = self.build_position
		building = self.unit_class(under_construction = True)
		game.game.world.add_unit(building, pos)
		WorkMission(building).begin(unit)

#-----------------------------------------------------------------------------

class WorkMission(TargetedMission):

	description = "Working on"
	
	def carry_out_on_target(self, unit, dt):
		building = self.get_target_unit()
		if building:
			if building.under_construction:
				building.perform_construction_work(unit, dt)
			else:
				self.mission_accomplished(unit)

			