#-----------------------------------------------------------------------------
#
#   Loon - Playing screen
#
#-----------------------------------------------------------------------------

from __future__ import division
from math import floor, ceil
from numpy import maximum, minimum
from pygame import draw, Color
from pygame.locals import *
from albow.widget import Widget
from albow.controls import Label, Button, ValueDisplay
from albow.palette_view import PaletteView
from albow.layout import Row, Column, Grid
from albow.references import Ref
from albow.resource import get_font, get_image
from albow.image_array import get_image_array
from humerus.main_commands import MainCommandScreen
from world_view import WorldView
from action_widget import ActionWidget
from unit_command_menu import UnitCommandMenu
from unit_building_menu import UnitBuildingMenu
from status_display import StatusDisplay
from colony_status_display import ColonyStatusDisplay
from unit_status_display import UnitStatusDisplay
from formations import BoxFormation
from game import game
from theme import UI
from constants import *

arrow_key_map = {K_LEFT: (-1, 0), K_RIGHT: (1, 0), K_UP: (0, -1), K_DOWN: (0, 1)}

#-----------------------------------------------------------------------------

class UIButton(UI, Button):
	pass

#-----------------------------------------------------------------------------

class UIBar(Widget):

	bg_color = Color("lightblue")

#-----------------------------------------------------------------------------

class PromptWidget(Label, ActionWidget):

	margin = 10

	def __init__(self, client):
		Label.__init__(self, self.prompt)
		self.client = client

#-----------------------------------------------------------------------------

class UnitSelectionWidget(PromptWidget):

	prompt = (
		"Click to select a unit\n\n"
		"Shift-click to select multiple units\n\n"
		"Double-click to select nearby units\n"
		"of the same type\n\n"
		"Shift-double-click to select all units\n"
		"of the same type"
	)
	
	def world_click(self, e):
		self.client.perform_unit_selection(e)

#-----------------------------------------------------------------------------

#class TargetSelectionWidget(PromptWidget):
#
#	prompt = "Click to choose target\nunit or location"
#	
#	def world_click(self, e):
#		self.client.perform_target_selection(e)

class TargetSelectionWidget(StatusDisplay, ActionWidget):

	prompt = "Click to choose target unit or location"
	
	def display_status(self):
		cmd = self.client.current_command
		if cmd:
			self.put_heading(cmd.description)
			self.leave_gap()
			self.put(self.prompt)
	
	def world_click(self, e):
		self.client.perform_target_selection(e)

#-----------------------------------------------------------------------------

#class SiteSelectionWidget(PromptWidget):
#
#	prompt = "Click to choose a\nbuilding site"
	
class SiteSelectionWidget(StatusDisplay, ActionWidget):

	prompt = "Click to choose a building site"
	mouse_position = None
	unit_class = None
	
	def display_status(self):
		cls = self.unit_class
		if cls:
			self.put_heading("Build " + cls.title)
			self.leave_gap()
			self.put(self.prompt)
	
	def world_mouse_move(self, e):
		self.mouse_position = e.local
	
	def world_click(self, e):
		self.client.perform_site_selection(e, self.unit_class)
	
	def draw_world_feedback(self, world_view, surf):
		p = self.mouse_position
		unit_class = self.unit_class
		if p is not None and unit_class is not None:
			world_view.draw_site_selection_feedback(surf, p, unit_class)

#-----------------------------------------------------------------------------

class PlayScreen(MainCommandScreen):
	#  action_widget      ActionWidget    UI for current phase of command
	#  current_command    UnitCommand

	ui_height = 150
	
	action_widget = None
	hscroll_state = 0
	vscroll_state = 0
	
	@property
	def world(self):
		return game.world
	
	@property
	def acting_unit(self):
		return self.world.first_selected_unit

	def __init__(self, shell):
		MainCommandScreen.__init__(self, shell)
		self.world_view = WorldView(shell.game, self)
		self.world_view.size = (self.width, self.height - self.ui_height)
		self.add(self.world_view)
		self.ui_bar = UIBar(width = self.width, height = self.ui_height)
		self.ui_bar.bottomleft = (0, self.height)
		self.add(self.ui_bar)
		self.unit_selection_prompt = UnitSelectionWidget(self)
		self.unit_command_menu = UnitCommandMenu(self)
		self.target_selection_prompt = TargetSelectionWidget(self)
		self.building_selection_menu = UnitBuildingMenu(self)
		self.site_selection_prompt = SiteSelectionWidget(self)
		self.cancel_button = UIButton("Cancel", action = self.cancel_command)
		self.cancel_button.bottomleft = (196, self.ui_bar.height - 10)
		self.ui_bar.add(self.cancel_button)
		self.colony_status = ColonyStatusDisplay(self, height = self.ui_height)
		self.colony_status.topright = (self.width, 0)
		self.ui_bar.add(self.colony_status)
		self.unit_status = UnitStatusDisplay(self, height = self.ui_height)
		self.unit_status.topright = self.colony_status.topleft
		self.ui_bar.add(self.unit_status)
		self.enter_unit_command_mode()
	
	def cancel_command(self):
		self.enter_unit_command_mode()
	
	def enter_unit_command_mode(self):
		world = self.world
		if world and world.selected_units:
			world.invalidate_unit_commands()
			widget = self.unit_command_menu
		else:
			widget = self.unit_selection_prompt
		self.set_current_command(None)
		self.show_action_widget(widget)
	
	def set_current_command(self, command):
		self.current_command = command
		self.cancel_button.visible = command is not None
	
	def enter_target_selection_mode(self, command):
		self.current_command = command
		self.show_action_widget(self.target_selection_prompt)
	
	def enter_building_selection_mode(self, command):
		self.current_command = command
		acting_unit = self.world.first_selected_unit
		menu = self.building_selection_menu
		menu.acting_unit = acting_unit
		world = self.world
		menu.buildable_units = [cls
			for cls in acting_unit.buildable_units
				if world.item_is_available(cls)]
		self.show_action_widget(menu)
	
	def enter_site_selection_mode(self, unit_class):
		prompt = self.site_selection_prompt
		prompt.unit_class = unit_class
		self.show_action_widget(prompt)
	
	def show_action_widget(self, widget):
		current = self.action_widget
		if widget is not current:
			if current:
				self.ui_bar.remove(current)
			self.action_widget = widget
			if widget:
				self.ui_bar.add(widget)
				widget.focus()
	
	def world_mouse_move(self, e):
		self.action_widget.world_mouse_move(e)

	def world_click(self, e):
		self.action_widget.world_click(e)
	
	def draw_world_feedback(self, world_view, surf):
		self.action_widget.draw_world_feedback(world_view, surf)
	
	def units_responding_to_command(self, cmd):
		for unit in self.world.selected_units:
			if unit.responds_to_command(cmd) and unit.command_available(cmd.name):
				yield unit

	def perform_unit_selection(self, e):
		self.world_view.perform_unit_selection(e)
		self.enter_unit_command_mode()

	def perform_target_selection(self, e):
		view = self.world_view
		p = e.local
		unit = view.find_unit_at_screen_point(p)
		cmd = self.current_command
		units = list(self.units_responding_to_command(cmd))
		if unit:
			target = unit
			for unit in units:
				cmd.invoke_on_unit_with_target(unit, target)
		else:
			centre = view.screen_to_world(p)
			#formation = BoxFormation(0.25, len(units), centre)
			formation = BoxFormation(units, centre)
			for unit, target in zip(units, formation):
				cmd.invoke_on_unit_with_target(unit, target)
		self.enter_unit_command_mode()

	def perform_site_selection(self, e, unit_class):
		view = self.world_view
		p = view.screen_to_world(e.local)
		acting_unit = self.acting_unit
		if acting_unit:
			self.current_command.invoke_on_unit_with_site_and_class(self, acting_unit, p, unit_class)
	
	def key_down(self, e):
		key = e.key
		if key == K_ESCAPE and self.current_command:
			self.cancel_command()
			return
		direction = arrow_key_map.get(key)
		if direction is not None:
			self.start_scrolling(*direction)
		else:
			MainCommandScreen.key_down(self, e)
	
	def key_up(self, e):
		key = e.key
		direction = arrow_key_map.get(key)
		if direction is not None:
			self.stop_scrolling(*direction)
		else:
			MainCommandScreen.key_up(self, e)
	
	def start_scrolling(self, h, v):
		if h:
			self.hscroll_state = h
		if v:
			self.vscroll_state = v
	
	def stop_scrolling(self, h, v):
		if self.hscroll_state == h:
			self.hscroll_state = 0
		if self.vscroll_state == v:
			self.vscroll_state = 0

	def timer_event(self, e):
		MainCommandScreen.timer_event(self, e)
		h = self.hscroll_state
		v = self.vscroll_state
		if h or v:
			world = self.world
			world_size = world.size * TILE_SIZE
			window_size = self.world_view.size
			max_offset = world_size - window_size
			d = SCROLL_RATE
			offset = world.scroll_offset + (d * h, d * v)
			offset = maximum(0, minimum(offset, window_size))
			world.scroll_offset = offset

