#-----------------------------------------------------------------------------
#
#   Loon - Resources
#
#-----------------------------------------------------------------------------

from numpy import zeros, minimum, maximum

resource_types_list = ['ore', 'water', 'silica', 'metal', 'glass']
resource_types = set(resource_types_list)

name_to_resource_index = {n:i for (i, n) in enumerate(resource_types_list)}

class ResourcePack(object):

	resources_accepted = resource_types
	resources_supplied = resource_types

	def __init__(self, resources_accepted = None, resources_supplied = None, max = None, **kwds):
		n = len(resource_types)
		self.amounts = zeros(n)
		self.max_amounts = zeros(n)
		if max is not None:
			self.max_amounts[:] = max
		self.update(kwds)
		self.max_amounts[:] = maximum(self.max_amounts, self.amounts)
		if resources_accepted is not None:
			self.resources_accepted = resources_accepted
		if resources_supplied is not None:
			self.resources_supplied = resources_supplied
	
	def update(self, kwds):
		for name in kwds:
			value = kwds[name]
			if name.startswith('max_'):
				a = self.max_amounts
				name = name[4:]
			else:
				a = self.amounts
			a[name_to_resource_index[name]] = value
	
	def reset_max(self, kwds):
		#print "ResourcePack.reset_max:", kwds ###
		a = self.max_amounts
		a[:] = kwds.get('max', 0)
		for name in kwds:
			if name.startswith('max_'):
				value = kwds[name]
				name = name[4:]
				a[name_to_resource_index[name]] = value
		for name in kwds:
			if name <> 'max' and not name.startswith('max_'):
				i = name_to_resource_index[name]
				value = kwds[name]
				a[i] = maximum(a[i], value)
	
	def full_of(self, name):
		i = name_to_resource_index[name]
		return self.amounts[i] >= self.max_amounts[i]
	
	def replete(self):
		return (self.amounts >= self.max_amounts).all()
	
	def add(self, name, amount):
		#  Attempt to add the given amount of the named resource.
		#  Returns the actual amount added.
		i = name_to_resource_index[name]
		amount = min(amount, self.max_amounts[i] - self.amounts[i])
		self.amounts[i] += amount
		return amount
	
	def transfer_to(self, other, **kwds):
		for name in kwds:
			i = name_to_resource_index[name]
			value = min(kwds[name], self.amounts[i], other.max_amounts[i])
			self.amounts[i] -= value
			other.amounts[i] += value
	
	def transfer_all_to(self, other):
		for name in self.resources_supplied & other.resources_accepted:
			i = name_to_resource_index[name]
			a = minimum(self.amounts[i], other.max_amounts[i] - other.amounts[i])
			self.amounts[i] -= a
			other.amounts[i] += a
	
	def convert(self, amount, from_type, to_type):
		#  Returns surplus amount
		#print "ResourcePack.convert:", amount, "from", from_type, "to", to_type ###
		i = name_to_resource_index[from_type]
		j = name_to_resource_index[to_type]
		a = min(amount, self.amounts[i], self.max_amounts[j] - self.amounts[j])
		self.amounts[i] -= a
		self.amounts[j] += a
		return amount - a

	def use(self, **kwds):
		#  Try to use the specified amounts. If all are available, uses them and
		#  returns None. Otherwise nothing is used and a dict of missing amounts
		#  is returned.
		missing = {}
		a = zeros(len(resource_types))
		for name in kwds:
			i = name_to_resource_index[name]
			amount = kwds[name]
			a[i] = min(amount, self.amounts[i])
			if a[i] < amount:
				missing[name] = amount - a[i]
		if missing:
			return missing
		self.amounts -= a

	def iter_contents(self):
		for i, name in enumerate(resource_types_list):
			a = self.amounts[i]
			m = self.max_amounts[i]
			if a or m:
				yield (name, a, m)
