#-----------------------------------------------------------------------------
#
#   Loon - Unit
#
#-----------------------------------------------------------------------------

from random import uniform
from numpy import ndarray, ceil, rint
from resources import ResourcePack
from vector_algebra import vector, unit, mag, rotate2, deg
import game
from constants import *

population_support_categories = ['housing', 'food', 'water']

class Unit(object):
	#  position        vector
	#  image           Image
	#  target          Unit or world point or None
	#  mission         Mission
	#  cooldown        float
	#  health          number
	#  resources       ResourcePack

	_position = vector(0.0, 0.0)
	radius = 0.5
	_target = None
	mission = None
	valid_multi_commands = set()  # Valid commands for multiple selection
	valid_single_commands = set() # Additional valid commands for single selection
	max_health = 10
	leaves_corpse = False # Not removed from world on death
	sight_range = 5.0 # Will move towards units in this range when target not set
	buildable_units = [] # Unit classes that this unit type is capable of building
	#time_to_build = 5 # seconds
	build_progress_fraction = None # 0.0 to 1.0 when building a unit
	obstructing = True # Prevents other units from overlapping it
	title = "Untitled Unit"
	health_label = "Health"
	is_alive = True
	under_construction = False
	disabled = False
	selection_priority = 0
	is_fixed = True
	resources = None
	initial_resources = None
	construction_resources = None
	resources_accepted = None
	resources_supplied = None
	population_support = None

	@classmethod
	def site_size_required(cls):
		s = int(ceil(2 * cls.radius))
		return vector(s, s)

	@classmethod
	def tile_is_suitable(cls, tile):
		return True
	
	@classmethod
	def time_to_build(cls):
		r = cls.radius
		return UNIT_BUILD_TIME_FACTOR * int(ceil(2 * r * r))

	@property
	def world(self):
		return game.game.world

	@property
	def busy(self):
		return self.is_busy()
	
	def is_busy(self):
		return False
		
	def get_position(self):
		return self._position
	
	def set_position(self, p):
		if not isinstance(p, ndarray):
			raise TypeError("position is not a vector")
		if len(p) <> 2:
			raise ValueError("Wrong dimension vector for position")
		self._position = p
	
	position = property(get_position, set_position)

	def get_target(self):
		return self._target
	
	def set_target(self, value):
		if isinstance(value, ndarray) and len(value) <> 2:
			raise ValueError("Wrong dimension vector for target")
		self._target = value
	
	target = property(get_target, set_target)

	def __init__(self, **kwds):
		self.__dict__.update(kwds)
		if self.under_construction:
			self.health = 0
			self.resources = ResourcePack(**self.construction_resources)
		else:
			self.health = self.max_health
			self.init_resources()
	
	def init_resources(self):
		if self.initial_resources:
			self.resources = ResourcePack(
				resources_accepted = self.resources_accepted,
				resources_supplied = self.resources_supplied,
				**self.initial_resources)
	
	def __setstate__(self, d):
		#print "Unit.__setstate__:", d ###
		self.__dict__.update(d)
		if not self.under_construction:
			if self.resources and self.initial_resources:
				self.resources.reset_max(self.initial_resources)
	
	def selection_ranking(self):
		return (self.selection_priority, self.position[1])

	def units_removed(self, units):
#		target = self.target
#		if isinstance(target, Unit) and target in units:
#			self.target = None
		if self.mission:
			self.mission.units_removed(self, units)
	
	def responds_to_command(self, cmd):
		name = cmd.name
		if name == 'cheat' and not options.enable_cheats_option:
			return False
		return name in self.valid_single_commands \
			or name in self.valid_multi_commands
	
	def perform_move_cmd(self,  target):
		pass
	
	def timestep(self, dt):
		#print "Unit.timestep:", dt ###
		if self.is_alive and not self.under_construction:
			self.live_timestep(dt)
	
	def live_timestep(self, dt):
		#print "Unit.live_timestep:", dt ###
		pass

	def take_damage(self, damage):
		health = max(0, self.health - damage)
		self.health = health
		if health <= 0:
			self.is_alive = False
	
	def heal(self, damage):
		self.health = min(self.health + damage, self.max_health)

	def command_available(self, name):
		return True

	def invalidate_unit_commands(self):
		self.world.invalidate_unit_commands()

	def activity_description(self):
		return None

	def draw(self, surf, rect):
		img = self.get_image()
		surf.blit(img, rect)
	
	def get_image(self):
		return self.image

	def get_image_rect(self):
		return self.get_image().get_rect()

	def covered_cells(self):
		r = self.radius
		x, y = self.position
		x0 = int(round(x - r))
		x1 = int(round(x + r))
		y0 = int(round(y - r))
		y1 = int(round(y + r))
		for x in xrange(x0, x1):
			for y in xrange(y0, y1):
				yield (x, y)

	def push_mission(self, mission):
		mission.next_mission = self.mission
		self.mission = mission

	def perform_construction_work(self, unit, dt):
		if self.under_construction and self.resources.replete():
			self.health = min(self.max_health, self.health + dt * unit.building_rate)
			if self.health >= self.max_health:
				self.under_construction = False
				self.init_resources()

	def display_status(self, status_display):
		self.display_activity(status_display)
		if self.under_construction:
			self.display_health(status_display)
		self.display_resources(status_display)
	
	def display_activity(self, status_display):
		activity = self.activity_description()
		if activity:
			status_display.put(*activity)
			status_display.leave_gap()
	
	def display_health(self, status_display):
		status_display.put(self.health_label, "%.0f / %.0f" % (self.health, self.max_health))
	
	def display_resources(self, status_display):
		res = self.resources
		if res:
			for name, amount, maximum in res.iter_contents():
				status_display.put(name.title(), "%.0f / %.0f" % (amount, maximum))
